(function () {
    return {
        createUI: function (namingContainer, container, settings, containerCallback, criteriaContainerDomNode) {
            var self = this;
            self.languageKeys = [
                "/shell/cms/visitorgroups/criteria/geographiclocation/continent",
                "/shell/cms/visitorgroups/criteria/geographiclocation/country",
                "/shell/cms/visitorgroups/criteria/geographiclocation/region",
                "/shell/cms/visitorgroups/criteria/geographiclocation/mustselectcontinent",
                "/shell/cms/visitorgroups/criteria/geographiclocation/validation/continentnotavailable",
                "/shell/cms/visitorgroups/criteria/geographiclocation/validation/countrynotavailable",
                "/shell/cms/visitorgroups/criteria/geographiclocation/validation/regionnotavailable"];

            function getMethodUrl(methodName) {
                return epi.routes.getActionPath({ moduleArea: "CMS", controller: "GeographicLocation", action: methodName });
            }

            // Fetch the translations from server, then we can generate the UI.
            var getLocationUrl = getMethodUrl("GetLocations");

            function getAvailableCapabilities() {
                return dojo.xhrGet({
                    url: getMethodUrl("GetAvailableCapabilities"),
                    handleAs: "json",
                    error: epi.cms.ErrorDialog.showXmlHttpError
                });
            }

            function validateSettings(regionsVisibility) {
                if (!settings) {
                    return null;
                }

                function setErrorState() {
                    //temporary
                    criteriaContainerDomNode.classList.add("error");
                }

                function isValueSelected(selection) {
                    return selection && selection !== "*";
                }

                var keyPrefix = "/shell/cms/visitorgroups/criteria/geographiclocation/validation/";

                // continent can't have 'any' value, just need to check if it's set
                if (settings.Continent && !regionsVisibility.showContinent) {
                    setErrorState();
                    return self.translatedText[keyPrefix + "continentnotavailable"] + " [" + settings.Continent + "]";
                }

                if (isValueSelected(settings.Country) && !regionsVisibility.showCountry) {
                    setErrorState();
                    return self.translatedText[keyPrefix + "countrynotavailable"] + " [" + settings.Country + "]";
                }

                if (isValueSelected(settings.Region) && !regionsVisibility.showRegion) {
                    setErrorState();
                    return self.translatedText[keyPrefix + "regionnotavailable"] + " [" + settings.Region + "]";
                }
            }

            getAvailableCapabilities().then(function (regionsVisibility) {
                this.getTranslation(function () {
                    var validationError = validateSettings(regionsVisibility);

                    var loadingComplete = false;
                    function loaded() {
                        loadingComplete = true;
                        var initializationErrors = validationError ? [validationError] : [];
                        containerCallback(namingContainer, container, undefined, initializationErrors);
                    }

                    if (!regionsVisibility.showContinent) {
                        loaded();
                        return;
                    }

                    var span = dojo.create("div", { "class": "epi-criteria-grouping" });
                    var element = dojo.create("label", { "class": "epi-criteria-label", "for": namingContainer + "continentSelect", innerHTML: self.translatedText["/shell/cms/visitorgroups/criteria/geographiclocation/continent"] + ": " });
                    dojo.place(element, span);
                    element = dojo.create("div");
                    element.id = namingContainer + "continentSelect";
                    dojo.place(element, span);
                    dojo.place(span, container);

                    if (regionsVisibility.showCountry) {
                        span = dojo.create("div", { "class": "epi-criteria-grouping" });
                        element = dojo.create("label", { "class": "epi-criteria-label", "for": namingContainer + "countrySelect", innerHTML: self.translatedText["/shell/cms/visitorgroups/criteria/geographiclocation/country"] + ": " });
                        dojo.place(element, span);
                        element = dojo.create("div");
                        element.id = namingContainer + "countrySelect";
                        dojo.place(element, span);
                        dojo.place(span, container);

                        if (regionsVisibility.showRegion) {
                            span = dojo.create("div", { "class": "epi-criteria-grouping" });
                            element = dojo.create("label", { "class": "epi-criteria-label", "for": namingContainer + "regionSelect", innerHTML: self.translatedText["/shell/cms/visitorgroups/criteria/geographiclocation/region"] + ": " });
                            dojo.place(element, span);
                            element = dojo.create("div");
                            element.id = namingContainer + "regionSelect";
                            dojo.place(element, span);
                            dojo.place(span, container);
                        }
                    }

                    dojo.require("dijit.form.FilteringSelect");

                    if (regionsVisibility.showCountry && regionsVisibility.showRegion) {
                        var regionSelection = new dijit.form.FilteringSelect(
                            {
                                id: namingContainer + "regionSelect",
                                name: namingContainer + "region",
                                searchAttr: "Key",
                                disabled: true,
                                onChange: function (item) {
                                },
                                selectOnClick: true
                            },
                            namingContainer + "regionSelect");
                        regionSelection.startup();
                    }

                    function loadRegions(settings) {
                        if (!regionsVisibility.showRegion) {
                            return [];
                        }

                        return dojo.xhrGet({
                            url: getLocationUrl + "?countryCode=" + settings.Country,
                            handleAs: "json",
                            error: epi.cms.ErrorDialog.showXmlHttpError
                        }).then(function (jsonData) {
                            var store = new dojo.data.ItemFileReadStore({ data: jsonData });
                            regionSelection.store = dojo.store.DataStore({ store: store });
                            regionSelection.set("disabled", false);
                            if (settings.Region) {
                                regionSelection.set("value", settings.Region);
                            } else {
                                if (settings.Region === null) {
                                    regionSelection.set("value", "", true, " ", null);
                                } else {
                                    regionSelection.set("value", "*");
                                }
                            }
                        });
                    }

                    var countrySelection;
                    if (regionsVisibility.showCountry) {
                        countrySelection = new dijit.form.FilteringSelect(
                            {
                                id: namingContainer + "countrySelect",
                                name: namingContainer + "country",
                                searchAttr: "Key",
                                disabled: true,
                                onChange: function (item) {
                                    if (!loadingComplete) {
                                        return;
                                    }
                                    if (settings) {
                                        settings.Region = undefined;
                                    }
                                    loadRegions({ Country: item });
                                },
                                selectOnClick: true
                            },
                            namingContainer + "countrySelect");
                        countrySelection.startup();
                    }

                    function loadCountries(settings) {
                        if (!regionsVisibility.showCountry) {
                            return [];
                        }

                        return dojo.xhrGet({
                            url: getLocationUrl + "?continentCode=" + settings.Continent,
                            handleAs: "json",
                            error: epi.cms.ErrorDialog.showXmlHttpError
                        }).then(function (jsonData) {
                            var store = new dojo.data.ItemFileReadStore({ data: jsonData });
                            countrySelection.store = dojo.store.DataStore({ store: store });
                            countrySelection.set("disabled", false);
                            if (settings.Country) {
                                countrySelection.set("value", settings.Country);
                            } else {
                                if (settings.Country === null) {
                                    countrySelection.set("value", "", true, " ", null);

                                } else {
                                    countrySelection.set("value", "*");
                                }
                            }
                            return loadRegions(settings);
                        });
                    }

                    var continentSelection = new dijit.form.FilteringSelect(
                        {
                            id: namingContainer + "continentSelect",
                            name: namingContainer + "continent",
                            searchAttr: "Key",
                            required: true,
                            missingMessage: self
                                .translatedText["/shell/cms/visitorgroups/criteria/geographiclocation/mustselectcontinent"],
                            onChange: function (item) {
                                if (!loadingComplete || !continentSelection.isValid()) {
                                    return;
                                }
                                if (settings) {
                                    settings.Country = undefined;
                                    settings.Region = undefined;
                                }
                                loadCountries({ Continent: item });
                            },
                            selectOnClick: true
                        }, namingContainer + "continentSelect");
                    continentSelection.startup();

                    dojo.xhrGet({
                        url: getLocationUrl,
                        handleAs: "json",
                        error: epi.cms.ErrorDialog.showXmlHttpError
                    }).then(function (jsonData) {
                        var store = new dojo.data.ItemFileReadStore({ data: jsonData });
                        continentSelection.store = dojo.store.DataStore({ store: store });

                        if (settings) {
                            if (settings.Continent) {
                                continentSelection.set("value", settings.Continent);
                            }
                            loadCountries(settings).then(function () {
                                // In IE/Firefox the onChange seems to be firing in the FilteringSelects after this
                                // has completed resetting the values to "any". Applying band-aid timeout
                                setTimeout(loaded, 0);
                            }).otherwise(function () {
                                if (settings.Region) {
                                    regionSelection.set("displayedValue", settings.Region);
                                }
                                loaded();
                            });
                        } else {
                            loaded();
                        }
                    }).otherwise(function () {
                        loaded();
                    });
                });
            }.bind(this));
        },

        validate: function (namingContainer) {
            var validationErrors = this.prototype.validate.apply(this, arguments);

            // when continent select control is not available
            // then it should be possible to save the criteria with all empty values.
            // Administrator has to remove the criteria
            var elementId = namingContainer + "continentSelect";
            if (!document.getElementById(elementId)) {
                validationErrors.Add(this.translatedText["/shell/cms/visitorgroups/criteria/geographiclocation/validation/continentnotavailable"], null);
            }

            return validationErrors;
        },

        getSettings: function (namingContainer) {
            function getSelectValue(selectName) {
                var elementId = namingContainer + selectName;
                if (!document.getElementById(elementId)) {
                    return null;
                }
                return dijit.byId(elementId).get("value");
            }

            return {
                Continent: getSelectValue("continentSelect"),
                Country: getSelectValue("countrySelect"),
                Region: getSelectValue("regionSelect")
            };
        }
    };
})();
